/* This tool was developed at the University of Delaware and is publicly
   available for academic and non-commercial use. University of Delaware
   assumes no responsibility for consequences arising from the use of
   this tool. It is released under the GPL license. All questions and
   feedback should be directed to Jelena Mirkovic <sunshine@cis.udel.edu> */

#include <stdio.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>

#include "libcoral.h"
#include "hashtab.h"

#define MAX_SOURCES 64

#define SERVER_STATS_HASH_SIZE 70000
#define CLIENT_STATS_HASH_SIZE 90000
#define COMMUNICATION_STATS_HASH_SIZE 90000

#define TRACE_EXPECTED_DURATION 7200 //two hours
#define INITIAL_DATA_LIST_SIZE 4096
#define TRACE_TIME_INTERVAL 300 //5-minute time interval to count active hosts

typedef struct communication_stats {
  struct client_stats *parent;

  uint32_t request_length; //in byte
  uint32_t response_length; //in byte

  int port_number; //client port number

  uint32_t tcp_seq; //TCP sequence number

  struct timespec time_stamp; //request time

  short communication_finalized; //0: not replied, 1: replied, 2: finalized

  struct communication_stats *next;
} communication_stats;

typedef struct client_stats {
  struct server_stats *parent;

  uint32_t ip_address;

  uint32_t request_count;

  struct communication_stats *com_list;

  struct client_stats *next;
} client_stats;

typedef struct server_stats {
  uint32_t ip_address;

  uint32_t request_count;
  uint32_t client_count;

  struct client_stats *clients;
} server_stats;


hash_tab *server_hash;
hash_tab *client_hash;
hash_tab *communication_hash;
int server_hash_count;
int client_hash_count;
int communication_hash_count;

//req frequency counter
uint32_t *req_per_second;
uint32_t current_time;
uint32_t clock_second_counter;

//structure to maintain the statistics data
double *data_list;
uint32_t data_list_counter;
uint32_t data_list_capacity;

